--------------------------------- Code by Tread ----- (Trealak on Steam) ---------------------------------
------------------------------- Developed For Tread's Fuel Types Framework -------------------------------
require "Vehicles/ISUI/ISVehicleMechanics"

--------------------------------------------------------
------------- Part Names in Mechanics Menu -------------

local ISVehicleMechanics_initParts = ISVehicleMechanics.initParts;
function ISVehicleMechanics:initParts(...)
    if not self.vehicle then return; end
    ISVehicleMechanics_initParts(self);
    for i,v in pairs(self.vehiclePart) do
        for j,k in ipairs(v.parts) do
            if k.part:isContainer() and k.part ~= self.vehicle:getPartById("GasTank") then
                local fuelType = k.part:getModData().RSFuelType
                if fuelType ~= nil then
                    k.name = getText("IGUI_VehiclePart" .. fuelType .. "StorageTankRS");
                    self:updateLayout()
                end
            elseif k.part:isContainer() and k.part == self.vehicle:getPartById("GasTank") then
                if k.part:getTable("multigastank") and k.part:getModData().RSFuelType ~= nil then
                    k.name = getText("IGUI_VehiclePart" .. k.part:getId()) .. " - " .. getText("IGUI_RSFuelType2_" .. k.part:getModData().RSFuelType);
                    self:updateLayout()
                else
                    k.name = getText("IGUI_VehiclePart" .. k.part:getId()) .. " - " .. getText("IGUI_RSFuelType2_" .. k.part:getContainerContentType());
                    self:updateLayout()
                end
            end
        end
    end
end

--------------------------------------------------------
------------ Part Details in Mechanics Menu ------------
local ISVehicleMechanics_renderPartDetail = ISVehicleMechanics.renderPartDetail;
function ISVehicleMechanics:renderPartDetail(part)
    local notSpecialGasTank = part:getId() == "GasTank" and not part:getTable("multigastank") --- check if part is not Dynamic Gas Tank
    if not part:getModData().RSFuelType or notSpecialGasTank  then --- for "normal" parts run native function
        ISVehicleMechanics_renderPartDetail(self, part)
    elseif part:getTable("multigastank") then               --- for special, dynamic Gas Tanks
        local FONT_HGT_SMALL = getTextManager():getFontHeight(UIFont.Small)
        local FONT_HGT_MEDIUM = getTextManager():getFontHeight(UIFont.Medium)
        local lineHgt = FONT_HGT_SMALL;
        local x = self.xCarTexOffset + (self.width - 10 - self.xCarTexOffset) / 2;
        local y = self:titleBarHeight() + 10 + FONT_HGT_MEDIUM + 5;
        local inventoryItem = part:getInventoryItem();
        if inventoryItem and part:getTable("install") then
            self:drawText(getText("IGUI_Item") .. ": " .. inventoryItem:getDisplayName(), x, y, 1, 1, 1, 1);
            y = y + lineHgt;
            self:drawText(getText("IGUI_char_Weight") .. ": " .. round(part:getInventoryItem():getWeight(),0), x, y, 1, 1, 1, 1);
            y = y + lineHgt;
            if instanceof(inventoryItem, "DrainableComboItem") then
                if round(inventoryItem:getUsedDelta() * 100, 2) < 5 then
                    self:drawText(getText("IGUI_invpanel_Remaining") .. ": " .. round(inventoryItem:getUsedDelta() * 100, 2) .. "%", x, y, 1, 0, 0, 1);
                else
                    self:drawText(getText("IGUI_invpanel_Remaining") .. ": " .. round(inventoryItem:getUsedDelta() * 100, 2) .. "%", x, y, 1, 1, 1, 1);
                end
                y = y + lineHgt;
            end
        elseif part:getItemType() and part:getTable("install") then
            self:drawText(getText("IGUI_Item") .. ": " .. getText("IGUI_Missing"), x, y, 1, 0, 0, 1);
            y = y + lineHgt;
        end
        local capacity = self:roundContainerContentAmount(part) .. " / " .. part:getContainerCapacity();
        if part:getItemType() and part:getInventoryItem() then
            if round(part:getContainerContentAmount(), 3) <= 0.1 then
                capacity = getText("Tooltip_outoffuel") .. " (0 / " .. part:getContainerCapacity() .. ")";
                self:drawText(capacity, x, y, 1, 0, 0, 1);
                y = y + lineHgt;
            else
                local label = getText("IGUI_Vehicle_ContainerCapacity_" .. part:getModData().RSFuelType, capacity)
                if round(part:getContainerContentAmount(), 2) < 5 then
                    self:drawText(label, x, y, 1, 0, 0, 1);
                else
                    self:drawText(label, x, y, 1, 1, 1, 1);
                end
                y = y + lineHgt;
            end
        end
    else                                                           --- else (StorageTanks) run my adjusted one
       -- local FONT_HGT_SMALL = getTextManager():getFontHeight(UIFont.Small)
        local FONT_HGT_MEDIUM = getTextManager():getFontHeight(UIFont.Medium)
       -- local lineHgt = FONT_HGT_SMALL;
        local x = self.xCarTexOffset + (self.width - 10 - self.xCarTexOffset) / 2;
        local y = self:titleBarHeight() + 10 + FONT_HGT_MEDIUM + 5;
        if part:getItemType() and part:getInventoryItem() then
            if part:getContainerContentType() and part:getModData().RSFuelType then ---------------------------------- This was added - Tread
                local capacity = self:roundContainerContentAmount(part) .. " / " .. part:getContainerCapacity();
                local label = getText("IGUI_Vehicle_ContainerCapacity_" .. part:getModData().RSFuelType, capacity)
                if round(part:getContainerContentAmount(), 2) < 5 then
                    self:drawText(label, x, y, 1, 0, 0, 1);
                else
                    self:drawText(label, x, y, 1, 1, 1, 1);
                end
            end
        end
    end
end
--------------------------------------------------------
------- Part Tooltips in Mechanics Menu Overlay --------
local ISVehicleMechanics_doPartContextMenu = ISVehicleMechanics.doPartContextMenu;
function ISVehicleMechanics:doPartContextMenu(part, x,y)
    ISVehicleMechanics_doPartContextMenu(self, part, x,y)
    if part:getTable("multigastank") and part:getItemType() and not part:getItemType():isEmpty() then --- Add check for Dynamic Gas Tank
        local playerObj = getSpecificPlayer(self.playerNum);
        if not part:getInventoryItem() then
            option = self.context:getOptionFromName(getText("IGUI_Install"))
            option.subOption = nil
            if part:getTable("install") then
                local fuelType = part:getModData().RSFuelType
              --  option = self.context:addOptionOnTop(getText("IGUI_Install"), playerObj, nil)
                if not ISVehicleMechanics.cheat and not part:getVehicle():canInstallPart(playerObj, part) then
                    option.notAvailable = true;
                    self:doMenuTooltip(part, option, "install", nil);
                else
                    local subMenu = ISContextMenu:getNew(self.context);
                    self.context:addSubMenu(option, subMenu);
                    local typeToItem = VehicleUtils.getItems(self.chr:getPlayerNum())
                    -- display all possible item that can be installed
                    for i=1,#RSGasTankItems[fuelType] do
                        local name
                        local fullname = RSGasTankItems[fuelType][i] .. part:getVehicle():getScript():getMechanicType();
                        local item = InventoryItemFactory.CreateItem( fullname);
                        if item then name = item:getName(); end
                        local itemOpt = subMenu:addOption(name, playerObj, nil);
                        self:doMenuTooltip(part, itemOpt, "install", fullname);
                        if not typeToItem[fullname] then
                            itemOpt.notAvailable = true;
                        else
                            -- display every item the player posess
                            local subMenuItem = ISContextMenu:getNew(subMenu);
                            self.context:addSubMenu(itemOpt, subMenuItem);
                            for j,v in ipairs(typeToItem[fullname]) do
                                local itemOpt2 = subMenuItem:addOption(v:getDisplayName() .. " (" .. v:getCondition() .. "%)", playerObj, ISVehiclePartMenu.onInstallPart, part, v);
                                self:doMenuTooltip(part, itemOpt2, "install", fullname);
                            end
                        end
                    end
                end
            end
        end
    end
end

------------------------------------------ Full overwrites below - Tread -----------------------------------------------
--------------------------------------------------------
------- Part Tooltips in Mechanics Menu Overlay --------
function ISVehicleMechanics:renderCarOverlayTooltip(partProps, part, carType, ...)
    if self.context and self.context:getIsVisible() then return false; end
    --	print(self:getMouseX(), self:getMouseY());
    if not self.tooltip then
        self.tooltip = ISToolTip:new();
        self.tooltip:initialise();
        self.tooltip:addToUIManager();
        self.tooltip.followMouse = true;
    end
    if self:isMouseOverPart(self:getMouseX(), self:getMouseY(), part) then
        self.tooltip:setVisible(true);
        self.tooltip:setName(getText("IGUI_VehiclePart" .. part:getId()));
        local inventoryItem = part:getInventoryItem()
        if (not inventoryItem and part:getTable("install")) then
            self.tooltip.description = "<RGB:1,0,0> " .. getText("IGUI_Missing");
        else
            self.tooltip.description = getText("IGUI_invpanel_Condition") .. ": " .. part:getCondition() .. "%";
            if part:getContainerContentType() then
                local text = "???"
                if part:getId() == "GasTank" and round(part:getContainerContentAmount(), 3) <= 0.1 and part:getTable("multigastank") and part:getModData().RSFuelType ~= nil then --- If empty Dynamic Gas Tank
                    self.tooltip:setName(getText("IGUI_VehiclePart" .. part:getId()) .. " - " .. getText("IGUI_RSFuelType2_" .. part:getModData().RSFuelType));
                    text = getText("Tooltip_outoffuel") .. " (0 / " .. part:getContainerCapacity() .. ")"
                elseif part:getId() == "GasTank" and round(part:getContainerContentAmount(), 3) <= 0.1 then --- If empty Gas Tank (regular)
                    self.tooltip:setName(getText("IGUI_VehiclePart" .. part:getId()) .. " - " .. getText("IGUI_RSFuelType2_" .. part:getContainerContentType()));
                    text = getText("Tooltip_outoffuel") .. " (0 / " .. part:getContainerCapacity() .. ")"
                elseif part:getModData().RSFuelType ~= nil and part:getId() ~= "GasTank" then --- Storage Tanks - Tread
                    local contents = self:roundContainerContentAmount(part) .. " / " .. part:getContainerCapacity()
                    self.tooltip:setName(getText("IGUI_VehiclePart" .. part:getModData().RSFuelType .. "StorageTankRS"));
                    text = getTextOrNull("IGUI_Vehicle_ContainerCapacity_" .. part:getModData().RSFuelType, contents)
                    if not text then
                        text = getText("IGUI_Vehicle_ContainerCapacity_Other", part:getModData().RSFuelType, contents)
                    end
                elseif part:getModData().RSFuelType == nil and part:getId() ~= "GasTank" then --- Storage Tanks (unknown content) - Tread
                    local contents = self:roundContainerContentAmount(part) .. " / " .. part:getContainerCapacity()
                    self.tooltip:setName(getText("IGUI_VehiclePart" .. part:getId()));
                    text = getTextOrNull("IGUI_Vehicle_ContainerCapacity_" .. part:getContainerContentType(), contents)
                    if not text then
                        text = getText("IGUI_Vehicle_ContainerCapacity_Other", part:getContainerContentType(), contents)
                    end
                elseif part:getTable("multigastank") and part:getModData().RSFuelType ~= nil then   --- if Dynamic Gas Tank
                    local contents = self:roundContainerContentAmount(part) .. " / " .. part:getContainerCapacity()
                    self.tooltip:setName(getText("IGUI_VehiclePart" .. part:getId()) .. " - " .. getText("IGUI_RSFuelType2_" .. part:getModData().RSFuelType));
                    text = getTextOrNull("IGUI_Vehicle_ContainerCapacity_" .. part:getModData().RSFuelType, contents)
                    if not text then
                        text = getText("IGUI_Vehicle_ContainerCapacity_Other", part:getModData().RSFuelType, contents)
                    end
                else    --- regular Gas Tank
                    local contents = self:roundContainerContentAmount(part) .. " / " .. part:getContainerCapacity()
                    self.tooltip:setName(getText("IGUI_VehiclePart" .. part:getId()) .. " - " .. getText("IGUI_RSFuelType2_" .. part:getContainerContentType()));
                    text = getTextOrNull("IGUI_Vehicle_ContainerCapacity_" .. part:getContainerContentType(), contents)
                    if not text then
                        text = getText("IGUI_Vehicle_ContainerCapacity_Other", part:getContainerContentType(), contents)
                    end
                end
                self.tooltip.description = self.tooltip.description .. " <LINE> " .. text
            elseif instanceof(inventoryItem, "DrainableComboItem") then
                local text = "???"
                text = getText("IGUI_invpanel_Remaining") .. ": " .. round(inventoryItem:getUsedDelta() * 100, 2) .. "%"
                self.tooltip.description = self.tooltip.description .. " <LINE> " .. text
            end
        end
        self.tooltip:bringToTop();
        return true;
    end
    return false;
end