--------------------------------- Code by Tread ----- (Trealak on Steam) ---------------------------------
------------------------------- Developed For Tread's Fuel Types Framework -------------------------------
require "Vehicles/Vehicles"

----------------------------------------------------------------------------------------------------------
-------------- Table defining possible Gas Tank items per fuel type - for Dynamic Gas Tanks --------------
RSGasTankItems = RSGasTankItems or {}
RSGasTankItems["Gasoline"] = {"Base.SmallGasTank","Base.NormalGasTank","Base.BigGasTank"}
RSGasTankItems["Diesel"] = {"Base.SmallGasTank","Base.NormalGasTank","Base.BigGasTank"}
RSGasTankItems["LPG"] = {"Base.SmallLPGTankRS","Base.NormalLPGTankRS","Base.BigLPGTankRS"}

----------------------------------------------------------------------------------------------------------
function Vehicles.Update.GasTank(vehicle, part, elapsedMinutes)  --- Full function overwrite
    local invItem = part:getInventoryItem();
    if not invItem then return; end
    local amount = part:getContainerContentAmount()
    if elapsedMinutes > 0 and amount > 0 and vehicle:isEngineRunning() then
        local amountOld = amount
        -- calcul how much gas is used, based mainly on engine speed, engine quality & mass.
        local gasMultiplier = 90000; --- Default value
    -------------------------- Fuel efficiency of fuel types --------------------------
        local fuelType = part:getContainerContentType() --- get Content Type of GasTank
        if part:getTable("multigastank") then fuelType = part:getModData().RSFuelType end --- Overwrite for Dynamic GasTank
        if fuelType == "Diesel" then
            gasMultiplier = 120000 --- less fuel consumption for Diesel
        elseif fuelType == "LPG" then
            gasMultiplier = 70000 --- more fuel consumption for LPG
        end
    -----------------------------------------------------------------------------------
        -- heater consume more gas
        local heater = vehicle:getHeater();
        if heater and heater:getModData().active then
            gasMultiplier = gasMultiplier - 5000;   --- There was an error here (vanilla), heater lowered fuel consumption. Swapped + into - to fix it.
        end
        -- if quality is 60, we do: 100 - 60 = 40; 40/2 = 20; 20/100=0.2; 0.2+1 = 1.2 : our multiplier;
        local qualityMultiplier = ((100 - vehicle:getEngineQuality()) / 200) + 1;
        local massMultiplier =  ((math.abs(1000 - vehicle:getScript():getMass())) / 300) + 1;
        -- the closer we are to change shift, the less we consume gas
        local speedToNextTransmission = ((vehicle:getMaxSpeed() / vehicle:getScript():getGearRatioCount()) * 0.71) * vehicle:getTransmissionNumber();
        local speedMultiplier = (speedToNextTransmission - vehicle:getCurrentSpeedKmHour()) * 350;
        -- if vehicle is stopped, we half the value of gas consummed
        if math.floor(vehicle:getCurrentSpeedKmHour()) > 0 then
            gasMultiplier = gasMultiplier / qualityMultiplier / massMultiplier;
        else
            gasMultiplier = (gasMultiplier / qualityMultiplier) * 2;
            speedMultiplier = 1;
        end
        -- we're at max gear, cap general gas consumption
        if speedMultiplier < 800 then
            speedMultiplier = 800;
        end


        --		local engineSpeed = math.min(vehicle:getEngineSpeed(), 6000)
        --		local engineSpeedCalc = 6000 - engineSpeed;

        --		local newAmount = engineSpeedCalc / gasMultiplier;
        local newAmount = (speedMultiplier / gasMultiplier)  * SandboxVars.CarGasConsumption;
        newAmount =  newAmount * (vehicle:getEngineSpeed()/2500.0);
        amount = amount - elapsedMinutes * newAmount;

        -- if your gas tank is in bad condition, you can simply lose fuel
        if part:getCondition() < 70 then
            if ZombRand(part:getCondition() * 2) == 0 then
                amount = amount - 0.01;
            end
        end

        part:setContainerContentAmount(amount, false, true);
        amount = part:getContainerContentAmount();
        local precision = (amount < 0.5) and 2 or 1
        if VehicleUtils.compareFloats(amountOld, amount, precision) then
            vehicle:transmitPartModData(part)
        end
    end
end

local VehicleUtils_createPartInventoryItem = VehicleUtils.createPartInventoryItem;
function VehicleUtils.createPartInventoryItem(part)
    if not part:getItemType() or part:getItemType():isEmpty() then return nil end
    local partContentType = part:getContainerContentType()
    if not part:getTable("multigastank") and not part:getTable("forcedfuel") then
        VehicleUtils_createPartInventoryItem(part)
    else
        local item;
        if not part:getInventoryItem() then
            local fuelType = partContentType
            if part:getTable("multigastank") then
                fuelType = RSFuelTypes.DetermineMultiGasTankFuel(nil, part)
            else --- so if "forcedfuel" table
                --fuelType = RSFuelTypes.DetermineFuelType(nil, part, partContentType)
                local vehicleName = part:getVehicle():getScript():getFullName();
                if RSForceFuelSwapCars[vehicleName] ~= nil then fuelType = RSForceFuelSwapCars[vehicleName]; end --- if was assigned by type, list entry will not exist
            end
            local v = part:getVehicle();
            --		if not part:isSpecificItem() then
            --TODO: If "forcedfuel" and part type not in default parts table - transcribe ItemType from script
            --TODO: For LPG force
            local chosenKey = ""
            for i=1,#RSGasTankItems[fuelType] do
                local fullname = RSGasTankItems[fuelType][i] .. part:getVehicle():getScript():getMechanicType();
                chosenKey = chosenKey .. fullname .. ';'
            end
            local itemType = v:getChoosenParts():get(chosenKey);
            if not itemType then
                for i=1,#RSGasTankItems[fuelType] do
                    if ZombRand(100) > (100 - (100/#RSGasTankItems[fuelType])) or i == #RSGasTankItems[fuelType] then
                        itemType = RSGasTankItems[fuelType][i] .. part:getVehicle():getScript():getMechanicType();
                        v:getChoosenParts():put(chosenKey, itemType);
                        break;
                    end
                end
            end
            item = InventoryItemFactory.CreateItem(itemType);
            local ScriptPartCapacity = part:getContainerCapacity()
            if ScriptPartCapacity and ScriptPartCapacity > 0 then --- Assign custom capacity (defined in veh/part template) to inventory item
                if fuelType == "LPG" then ScriptPartCapacity = math.ceil(ScriptPartCapacity * 0.8) end --- LPG gets 80% capacity
                item:setMaxCapacity(ScriptPartCapacity);
            end
            part:setInventoryItem(item)
            part:setRandomCondition(item);
        end
    end
    return part:getInventoryItem()
end