--------------------------------- Code by Tread ----- (Trealak on Steam) ---------------------------------
------------------------------- Developed For Tread's Fuel Types Framework -------------------------------
------ inspired by FuelAPI and other mods by Konijima - most of functions or copies of FuelAPI ones ------

--------------------------------------- Sandbox Settings functions ---------------------------------------
UtilsRS = {};

function UtilsRS.CarFuelTypesSwap() --- Can Propane Tank items be used to fuel/siphon vehicles?
local option = 1; --- Default value in case Sandbox-options.txt does not load etc.
    if SandboxVars.RSFuelTypesAPI then
        option = tonumber(SandboxVars.RSFuelTypesAPI.CarFuelTypesSwap); --- 1 for Vehicle Type + List, 2 for Vehicle Type, 3 for List, 4 for Vanilla
    end
    return option;
end

function UtilsRS.PropaneOnGasStations() --- Can Propane Tank items be used to fuel/siphon vehicles?
local option = 1; --- Default value in case Sandbox-options.txt does not load etc.
    if SandboxVars.RSFuelTypesAPI then
        option = tonumber(SandboxVars.RSFuelTypesAPI.PropaneOnGasStations); --- 1 for Available, 2 for Not Available
    end
    return option;
end

function UtilsRS.PropaneTankItemFuelling() --- Can Propane Tank items be used to fuel/siphon vehicles?
local option = 1; --- Default value in case Sandbox-options.txt does not load etc.
    if SandboxVars.RSFuelTypesAPI then
        option = tonumber(SandboxVars.RSFuelTypesAPI.PropaneTankItemFuelling); --- 1 for Not Allowed, 2 for Allowed
    end
    return option;
end

--------------- Those functions are copies or heavily base on KONIJIMA's FuelAPI functions ---------------
if not getActivatedMods():contains("FuelAPI") then --- Use those functions if FuelAPI is off or missing
    local function SplitString(str, delimiter)
        local result = {};
        for match in (str..delimiter):gmatch("(.-)%"..delimiter) do
            table.insert(result, match);
        end
        return result;
    end

    local Utils = {};

    function Utils.IsCustom(item)
        return item:getTags():contains("CustomFuelContainer");
    end

    ---@param item InventoryItem
    function Utils.PredicateNotEmpty(item)
        return Utils.IsCustom(item) and instanceof(item, "DrainableComboItem") and item:getUsedDelta() > 0;
    end
--[[ FuelAPI function - not used by my mod
    ---@param item InventoryItem
    function Utils.PredicateNotEmptyWithBase(item)
        return Utils.PredicateNotEmpty(item) or (item:getFullType() == "Base.PetrolCan" and item:getUsedDelta() > 0);
    end
]]--
    ---@param item InventoryItem
    function Utils.PredicateNotFull(item)
        return Utils.IsCustom(item) and instanceof(item, "DrainableComboItem") and item:getUsedDelta() < 1;
    end
--[[ FuelAPI function - not used by my mod
    ---@param item InventoryItem
    function Utils.PredicateNotFullWithBase(item)
        return Utils.PredicateNotFull(item) or item:getFullType() == "Base.PetrolCan" and item:getUsedDelta() < 1;
    end
]]--
    ---@param item InventoryItem
    function Utils.PredicateEmpty(item)
        return Utils.IsCustom(item) and not instanceof(item, "DrainableComboItem");
    end

    ---@param item InventoryItem
    function Utils.PredicateEmptyWithBase(item)
        return Utils.PredicateEmpty(item) or item:getFullType() == "Base.EmptyPetrolCan";
    end

    ---@param item DrainableComboItem
    function Utils.GetProperLitres(item)
        local customCapacity = 1 / item:getUseDelta();
        local diff = customCapacity / 10;  --- changed 8 into 10 (as newest jerrycan capacity is described in game files)
        return Vehicles.JerryCanLitres * diff;
    end
----------------------------------Tread ------------ changed
    ---@param item InventoryItem
    function Utils.GetItemFromTypeTag(item, fuelType) --- redone grabbing by petrol tag into universal one
        local tags = item:getTags();
        for i=0, tags:size()-1 do
            local tag = tags:get(i);
            local splitted = SplitString(tag, "_");
            if splitted and #splitted == 3 and splitted[1] == fuelType then
                return splitted[2] .. "." .. splitted[3];
            end;
        end
    end

    ---------------------------------Tread --------- additions
    function Utils.GetItemFuelType(item) --- Check Item Fuel Type (by tags) - Tread
        local tags = item:getTags();
        for i=0, tags:size()-1 do
            local tag = tags:get(i);
            local splitted = SplitString(tag, "_");
            if splitted and #splitted == 2 and splitted[1] == "FuelType" then
                return splitted[2];
            end;
        end
        return false;
    end

    function Utils.GetFirstFuelMatch(itemsList, fuelType)
        for _, item in ipairs(itemsList) do
            if Utils.GetItemFuelType(item) == fuelType then return item end
        end
        return false
    end

    ---@param item InventoryItem
    function Utils.PredicateNotEmptyPetrol(item)
        return Utils.IsCustom(item) and instanceof(item, "DrainableComboItem") and item:getUsedDelta() > 0 and Utils.GetItemFuelType(item) == "Gasoline";
    end

    function Utils.FilterItemsByTag(itemList, Tag)
        local result = {}
        for i = 0, itemList:size() - 1 do
            ---@type InventoryItem
            local item = itemList:get(i)
            if item:getTags():contains(Tag) then
                table.insert(result, item)
            end
        end
        return result
    end

    function Utils.PredicatePropaneTankNotEmpty(item)
        return item:hasTag("FuelType_LPG") and item:getUsedDelta() > 0;
    end

    function Utils.PredicatePropaneTankNotFull(item)
        return item:hasTag("FuelType_LPG") and item:getUsedDelta() < 1;
    end

    return Utils, UtilsRS;
end
return UtilsRS;